import argparse
import os
from pathlib import Path
import sys

from .parser import MinifyMDT  # assuming your class is saved as `minify_mdt.py`


def transpile_file(in_path: Path, out_path: Path, layout: str, minify: bool):
    with in_path.open("r", encoding="utf-8") as f:
        md = f.read()
    parser = MinifyMDT(markdown_string=md, layout=layout, minify=minify)
    result = parser.transform()
    with out_path.open("w", encoding="utf-8") as f:
        f.write(result)


def create_parser():
    parser = argparse.ArgumentParser(description="Transpile Markdown tables to JSON code blocks.")
    parser.add_argument("-f", "--file", help="Single markdown file to transpile")
    parser.add_argument("-k", "--out-file", help="Output file for the transpiled JSON")
    parser.add_argument("-d", "--dir", help="Directory of markdown files to transpile")
    parser.add_argument("-o", "--out", help="Output directory for the transpiled JSON files")
    parser.add_argument("-l",
                        "--layout",
                        choices=["SoA", "AoS"],
                        default="SoA",
                        help="Layout of JSON output")
    parser.add_argument("-m", "--minify", action="store_true", help="Minify JSON output")

    args = parser.parse_args()

    # Validation
    if not args.file and not args.dir:
        print("[-] No file or directory provided")
        sys.exit(1)
    if not args.out_file and not args.out:
        print("[-] No output file or output directory provided")
        sys.exit(1)
    if args.file and args.dir:
        print("[-] Cannot provide both a file and a directory")
        sys.exit(1)
    if args.out_file and args.out:
        print("[-] Cannot provide both an output file and output directory")
        sys.exit(1)

    print(f"[+] Transpiling using layout: {args.layout}")

    if args.file:
        in_path = Path(args.file)
        out_path = Path(args.out_file)
        transpile_file(in_path, out_path, layout=args.layout, minify=args.minify)

    elif args.dir:
        in_dir = Path(args.dir)
        out_dir = Path(args.out)
        out_dir.mkdir(parents=True, exist_ok=True)
        for file_path in in_dir.glob("*.md"):
            out_path = out_dir / f"json_{file_path.name}"
            transpile_file(file_path, out_path, layout=args.layout, minify=args.minify)



import json
import re


class MinifyMDT:

    def __init__(self, markdown_string, layout="SoA", minify=True):
        self.markdown_string = markdown_string
        self.layout = layout
        self.minify = minify

    def _table_to_json(self, header, rows):
        if self.layout == "AoS":
            return [{header[i]: row[i] for i in range(len(header))} for row in rows]
        else:  # SoA
            return {col: [row[i] for row in rows] for i, col in enumerate(header)}

    def transform(self):
        table_pattern = re.compile(
            r"((?:\|[^\n]*\|[^\n]*\n)+\|[ \t]*[-:]+[^\n]*\n((?:\|[^\n]*\|[^\n]*\n?)+))",
            re.MULTILINE)

        def table_replacer(match):
            table_text = match.group(0).strip()
            lines = [line.strip() for line in table_text.splitlines()]
            header = [cell.strip() for cell in lines[0].strip("|").split("|")]
            rows = [
                [cell.strip() for cell in line.strip("|").split("|")] for line in lines[2:] if line
            ]
            json_obj = self._table_to_json(header, rows)
            json_str = json.dumps(json_obj,
                                  separators=(",", ":") if self.minify else None,
                                  indent=None if self.minify else 2)
            return f"```json\n{json_str}\n```"

        return table_pattern.sub(table_replacer, self.markdown_string)


if __name__ == "__main__":
    from pathlib import Path
    md_sample_path = Path("pymdt2json", "tests", "assets", "small_sample.md")
    assert md_sample_path.exists()
    with md_sample_path.open("r") as file:
        md_text = file.read()
    parser = MarkdownTable2Json(markdown_string=md_text, layout="AoS", minify=True)
    result = parser.transform()
    print(result)



from setuptools import setup, find_packages

setup(
    name="pymdt2json",
    version="0.1.0",
    description="Convert markdown tables into JSON code blocks",
    author="Amadou Wolfgang Cisse",
    author_email="amadou.6e@googlemail.com",
    url="https://github.com/amadou-6e/pymdt2json.git",  # change this
    packages=find_packages(),  # looks in current dir
    entry_points={
        "console_scripts": ["pymdt2json=pymdt2json:main",],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    include_package_data=True,
)
